VERSION 5.00
Begin VB.Form Bezout 
   Caption         =   "Compute GCD, LCM and Bezout equality"
   ClientHeight    =   4695
   ClientLeft      =   60
   ClientTop       =   630
   ClientWidth     =   5520
   LinkTopic       =   "Form1"
   ScaleHeight     =   4695
   ScaleWidth      =   5520
   StartUpPosition =   3  'Windows Default
   Begin VB.Timer checkPosition 
      Enabled         =   0   'False
      Interval        =   5000
      Left            =   75
      Top             =   2955
   End
   Begin VB.CommandButton cmdExit 
      Cancel          =   -1  'True
      Caption         =   "E&xit"
      Height          =   450
      Left            =   60
      TabIndex        =   5
      Top             =   4170
      Width           =   5325
   End
   Begin VB.TextBox txtB 
      Height          =   360
      Left            =   855
      TabIndex        =   3
      Top             =   645
      Width           =   2010
   End
   Begin VB.TextBox txtA 
      Height          =   360
      Left            =   855
      TabIndex        =   2
      Top             =   120
      Width           =   1995
   End
   Begin VB.CommandButton cmdCompute 
      Default         =   -1  'True
      Height          =   900
      Left            =   3105
      TabIndex        =   0
      Top             =   120
      Width           =   2025
   End
   Begin VB.Label lblResult 
      Height          =   2850
      Left            =   255
      TabIndex        =   6
      Top             =   1200
      Width           =   5130
   End
   Begin VB.Label Label2 
      Caption         =   "b = "
      Height          =   315
      Left            =   255
      TabIndex        =   4
      Top             =   645
      Width           =   375
   End
   Begin VB.Label Label1 
      Caption         =   "a = "
      Height          =   315
      Left            =   255
      TabIndex        =   1
      Top             =   270
      Width           =   375
   End
   Begin VB.Menu mnuQuit 
      Caption         =   "&Quit"
   End
End
Attribute VB_Name = "Bezout"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False

Option Explicit

' COMPUTE THE GCD AND LCM OF TWO NUMBERS
' TOGETHER WITH A BEZOUT EQUALITY

' The rationale is to use the Euclides algorithm
' and to memorize the results in a 2x2 Matrix Dn
' such that
'    a            Rn-1
'          = Dn .
'    b            Rn-2
'
' if Rn = Qn x Rn-1  + Rn+1
' then Dn+1 = Dn . Q  , where
'
'                 Qn      1
'    Q     =
'                 1       0
' (Qo being the identity matrix)
'
' For performance reasons the matrix D is kept in variables d1 through d4

Const computeCaption As String = "&Compute GCD(a,b)," & vbCr _
    & "LCM(a,b) and" & vbCr _
    & "Bezout relationship"
Const newValCaption = "Enter &new" & vbCr & "values"

' the numbers whose GDC and LCM are wanted
Dim A As Long, B As Long

' the matrix and temp storage variables
Dim d1 As Long, d2 As Long, d3 As Long, d4 As Long
Dim dPrim1 As Long, dPrim3 As Long
Dim det As Long ' determinant of D

' the inverse of matrix D
Dim b1 As Long, b2 As Long
' Dim b3 As Long, b4 As Long

' the variables for Euclides algorithm
Dim X As Long, Y As Long, Q As Long, R As Long
Dim LCM As Double

' miscellaneous
Dim sizW As Long, sizH As Long
Const minMargin As Long = 200
Const timerTick As Long = 1000 ' 5 seconds

Private Sub cmdCompute_Click()
If cmdCompute.Caption = computeCaption Then
  If txtA.Text = "" Then
    txtA.SetFocus: Exit Sub
  Else
    If txtB.Text = "" Then txtB.SetFocus: Exit Sub
  End If
' compute GCD, etc.
  txtA.Enabled = False
  txtB.Enabled = False
  If getAandB Then
    cmdCompute.Enabled = False
' initialize the matrix to Identity
    d1 = 1: d2 = 0: d3 = 0: d4 = 1
' set A as larger of the numbers
    If A < B Then R = A: A = B: B = R
    X = A: Y = B: Q = 0: R = A
    txtA.Text = CStr(A)
    txtB.Text = CStr(B)
    det = 1
' Euclides algorithm is just in the 6 lines below !
    Do While R <> 0
      euclDiv
      multMat
      X = Y
      Y = R
    Loop
' Here we've got (A,B) = D . (X,0) and X = GCD (A,B)
' and A = d1 * X  ,  B = d3 * X  ,  LCM(A,B) = d1 * d3 * X

' we inverse the matrix (just as much as necessary)
' to get a Bezout relationship
    invMat
    lblResult.Caption = "GCD(" & CStr(A) & "," & CStr(B) & ") = " & CStr(X) & vbCr
    If X = 1 Then
    lblResult.Caption = lblResult.Caption & _
      "(" & CStr(A) & " and " & CStr(B) & " are relatively prime)" & vbCr
    End If
    lblResult.Caption = lblResult.Caption & _
      CStr(A) & " = " & CStr(d1) & " * " & CStr(X) & vbCr & _
      CStr(B) & " = " & CStr(d3) & " * " & CStr(X) & vbCr
' we have GCD(A,B) = X = b1 * A  +  b2 * B
' we check it if we can (using the LCM variable for capacity reasons)
    On Error Resume Next
    LCM = b1 * A + b2 * B
    If Err.Number = 0 Then
      If LCM <> X Then
        MsgBox "Fatal error #03 - Must quit", vbCritical, "Fatal error #03 in Bezout"
        End
      End If
    End If
    Err.Clear
    LCM = d1 * d3 * X ' LCM(a,b)
    If Err.Number = 0 Then
      lblResult.Caption = lblResult.Caption & vbCr & _
        "LCM(" & CStr(A) & "," & CStr(B) & ") = " & CStr(LCM) & vbCr & _
        CStr(LCM) & " = " & CStr(A) & " * " & CStr(d3) & vbCr & _
        CStr(LCM) & " = " & CStr(B) & " * " & CStr(d1)
    Else
      lblResult.Caption = lblResult.Caption & vbCr & _
        "LCM(" & CStr(A) & "," & CStr(B) & ") = " & CStr(A) & " * " & CStr(d3) & vbCr & _
        "LCM(" & CStr(A) & "," & CStr(B) & ") = " & CStr(B) & " * " & CStr(d1)
    End If
    Err.Clear
    On Error GoTo 0
    lblResult.Caption = lblResult.Caption & vbCr & _
      vbCr & "Bezout equality:" & vbCr & CStr(X) & " = "
    If b1 >= 0 Then
      lblResult.Caption = lblResult.Caption & CStr(b1)
    Else
      lblResult.Caption = lblResult.Caption & "(-" & CStr(-b1) & ")"
    End If
    lblResult.Caption = lblResult.Caption & " * " & CStr(A) & "  +  "
    If b2 >= 0 Then
      lblResult.Caption = lblResult.Caption & CStr(b2)
    Else
      lblResult.Caption = lblResult.Caption & "(-" & CStr(-b2) & ")"
    End If
    lblResult.Caption = lblResult.Caption & " * " & CStr(B)
    cmdCompute.Caption = newValCaption
    cmdCompute.Enabled = True
    cmdCompute.SetFocus
  Else
    txtA.Enabled = True
    txtB.Enabled = True
  End If

' click on the "enter new value button"
ElseIf cmdCompute.Caption = newValCaption Then
  txtA.Enabled = True
  txtA.Text = ""
  txtB.Enabled = True
  txtB.Text = ""
  lblResult.Caption = ""
  cmdCompute.Caption = computeCaption
  txtA.SetFocus
Else
  MsgBox "Fatal error #01 - Must quit", vbCritical, "Fatal error #01 in Bezout"
  End
End If
End Sub

' inversion of matrix D
Private Sub invMat()
' the determinant of D must be 1 or -1
' as D is a product of matrices whose determinant is -1
' det = (d1 * d4) - (d2 * d3) has already been computed
If (det = 1) Or (det = -1) Then
  b1 = d4 * det
  b2 = (-d2) * det
' we only need b1 and b2
'  b3 = (-d3) / det
'  b4 = d1 / det
Else
  MsgBox "Fatal error #02 - Must quit", vbCritical, "Fatal error #02 in Bezout"
  End
End If
End Sub

' multiply matrix by result of division
Private Sub multMat()
dPrim1 = Q * d1 + d2
dPrim3 = Q * d3 + d4
d2 = d1
d4 = d3
d1 = dPrim1
d3 = dPrim3
det = -det ' because det(Q) = -1
End Sub

' do the Euclidean division of X by Y
Private Sub euclDiv()
Q = X \ Y
R = X - Q * Y
End Sub

' get the numbers
Private Function getAandB() As Boolean
Dim bb As Boolean
Dim s As String

bb = False
A = getVal(txtA.Text)
If A < 1 Then
  MsgBox "Illegal value of 'a'", vbInformation, "Please re-enter value of 'a'"
  txtA.Enabled = True
  txtA.SelStart = 0
  txtA.SelLength = Len(txtA.Text)
  txtA.SetFocus
Else
  B = getVal(txtB.Text)
  If B < 1 Then
    MsgBox "Illegal value of 'b'", vbInformation, "Please re-enter value of 'b'"
    txtB.Enabled = True
    txtB.SelStart = 0
    txtB.SelLength = Len(txtB.Text)
    txtB.SetFocus
  Else
    bb = True
  End If
End If
getAandB = bb
End Function

' get value of a number
Private Function getVal(s As String) As Long
Dim i As Integer, j As Integer, l As Long
Dim ascVal As Integer

l = Len(s)
If l > 11 Then
  getVal = -1
  Exit Function
End If
If Left(s, 1) = "-" Then
  i = 2
Else
  i = 1
End If
For j = i To l
  ascVal = Asc(Mid(s, j, 1))
  If ascVal < Asc("0") Or ascVal > Asc("9") Then
    getVal = -1
    Exit Function
  End If
Next
On Error Resume Next
getVal = CLng(Mid(s, i, l))
If Err.Number <> 0 Then
  getVal = -1
  Err.Clear
End If
End Function

Private Sub txtA_GotFocus()
txtA.SelStart = 0
txtA.SelLength = Len(txtA.Text)
End Sub

Private Sub txtB_GotFocus()
txtB.SelStart = 0
txtB.SelLength = Len(txtB.Text)
End Sub

Private Sub cmdExit_Click()
End
End Sub

Private Sub Form_Load()
sizW = Me.Width
sizH = Me.Height
Me.Move (Screen.Width - sizW) / 2, (Screen.Height - sizH) / 2
End Sub

Private Sub Form_Activate()
cmdCompute.Caption = computeCaption
txtA.Text = ""
txtB.Text = ""
txtA.SetFocus
checkPosition.Interval = timerTick
checkPosition.Enabled = True
End Sub

Private Sub Form_Resize()
Dim tp As Long, lft As Long

tp = Me.Top: If tp < 100 Then tp = 100
lft = Me.Left: If lft < 100 Then lft = 100

Select Case Me.WindowState ' selon nouvel etat
  Case vbMinimized ' "Move" forbidden in this case
  Case vbMaximized
    Me.WindowState = vbNormal ' wait for the next "resize" envent to do the Move
  Case vbNormal
    Me.Move lft, tp, sizW, sizH
End Select
End Sub

Private Sub checkPosition_Timer()
Dim tp As Long, lft As Long

tp = Me.Top
If tp < minMargin Then tp = minMargin
If tp > (Screen.Height - sizH - minMargin) Then tp = Screen.Height - sizH - minMargin
lft = Me.Left
If lft < minMargin Then lft = minMargin
If lft > (Screen.Width - sizW - minMargin) Then lft = Screen.Width - sizW - minMargin

Select Case Me.WindowState ' selon nouvel etat
  Case vbMinimized ' "Move" forbidden in this case
  Case vbMaximized
    Me.WindowState = vbNormal ' wait for the next "resize" envent to do the Move
  Case vbNormal
    Me.Move lft, tp, sizW, sizH
End Select
End Sub

Private Sub mnuQuit_Click()
End
End Sub

